==> interzone-1.0.23.tar.gz: lib/moderator/ModeratorCapsule.dart <==
import 'dart:typed_data';
import 'ModeratorCapsuleEntry.dart';

enum capsuleFieldType {
  empty,
  tangle,
  katamari,
  threadEntries,
  groupEntries,
  xxWant,
  xxData
}

class ModeratorCapsule {
  //Uint8List tangle; //MUST have token for POST
  Uint8List katamari; //OPTIONAL token for POST
  Uint8List threadEntries; //singular thread entries
  Uint8List groupEntries; //group entries pointing to thread entries
  Uint8List xxWants; //uint64's of missing xx
  Uint8List xxData; //bin data of xx to satisfy
  //for stripping dup entries
  bool get isEmpty =>
      katamari.isEmpty &&
      threadEntries.isEmpty &&
      xxWants.isEmpty &&
      xxData.isEmpty;
  bool get isDuplicate =>
      katamari.lengthInBytes == 1 &&
      threadEntries.lengthInBytes == 1 &&
      groupEntries.lengthInBytes == 1;

  ModeratorCapsule(this.katamari, this.threadEntries, this.groupEntries,
      this.xxWants, this.xxData);
  factory ModeratorCapsule.duplicate() {
    return ModeratorCapsule(
        Uint8List(1), Uint8List(1), Uint8List(1), Uint8List(1), Uint8List(1));
  }
  factory ModeratorCapsule.fromBinary(Uint8List data) {
    var bdata = new ByteData.view(data.buffer);
    data.buffer.asUint8List().toList();
    int dataLength = bdata.getUint32(0);
    var wi = 6; //skipping tangloid
    Uint8List _tangle = Uint8List(0); //MUST have key to talk to moderator
    Uint8List _katamari =
        Uint8List(dataLength); //currently OPTIONAL key for your requests
    Uint8List _threadEntries =
        Uint8List(10065536); //single thread entries with keys
    Uint8List _groupEntries =
        Uint8List(10065536); //single group entries with keys
    Uint8List _xxWants = Uint8List(8089); //single group entries with keys
    Uint8List _xxData =
        Uint8List(data.buffer.lengthInBytes); //single group entries with keys

    //tangle is a must
    /*int tadataLength = 0;
    if (bdata.getInt8(wi) == capsuleFieldType.empty) {
      wi++;
    } else {
      wi++;
      tadataLength = bdata.getInt16(wi);
      wi = wi + 2;
      for (int i = 0; i < tadataLength; i++) {
        _tangle.add(data[wi + i]);
      }
    }*/
    //katamari
    int totle = bdata.getUint32(0);
    if (totle > bdata.lengthInBytes) {
      return ModeratorCapsule(
          Uint8List(0), Uint8List(0), Uint8List(0), Uint8List(0), Uint8List(0));
    }
    print(totle.toString());
    print(bdata.lengthInBytes.toString());
    int kdataLength = 0;
    if (bdata.getUint8(wi) == capsuleFieldType.empty.index) {
      wi++;
    } else {
      wi++;
      kdataLength = bdata.getUint16(wi);
      wi = wi + 2;
      var moo = wi + kdataLength;
      //print(moo);
      //print(bdata.lengthInBytes);
      try {
        for (int i = 0; i < kdataLength; i++) {
          //_katamari.add(data[wi + i]);
          _katamari[i] = bdata.getUint8(wi + i);
        }
      } catch (e) {
        print('missing byte');
        print(moo);
        print(bdata.lengthInBytes);
      }
      wi += kdataLength;
    }
    //threads 5
    var tdataLength = 0;
    if (bdata.getUint8(wi) == capsuleFieldType.empty.index) {
      wi++;
    } else {
      wi++;
      tdataLength = bdata.getUint16(wi);
      if (tdataLength > bdata.lengthInBytes) {
        print('trap');
      }
      if (tdataLength > _threadEntries.lengthInBytes) {
        print('trap');
      }
      wi = wi + 2;
      for (int i = 0; i < tdataLength; i++) {
        _threadEntries[i] = bdata.getUint8(wi + i);
        //_threadEntries.add(data[wi + i]);
      }
      wi += tdataLength;
    }
    //groups 6
    if (bdata.getUint8(wi) == capsuleFieldType.empty.index) {
      wi++;
    } else {
      wi++;
      int xdataLength = bdata.getUint16(wi);
      wi = wi + 2;
      for (int i = 0; i < xdataLength; i++) {
        _groupEntries[i] = bdata.getUint8(wi + i);
      }
      wi += xdataLength;
    }
    //XXWANTS 7
    int xwdataLength = 0;
    if (wi + 8 > bdata.lengthInBytes) {
      return ModeratorCapsule(
          kdataLength != 0 ? _katamari.sublist(0, kdataLength) : Uint8List(0),
          _threadEntries.sublist(0, tdataLength),
          Uint8List(0),
          Uint8List(0),
          Uint8List(0));
    }
    if (bdata.getUint8(wi) == capsuleFieldType.empty.index) {
      wi++;
    } else {
      wi++;
      xwdataLength = bdata.getUint16(wi);
      wi = wi + 2;
      for (int i = 0; i < xwdataLength; i++) {
        _xxWants[i] = bdata.getUint8(wi + i);
      }
      wi += xwdataLength;
    }
    int xdataLength = 0;
    if (bdata.getUint8(wi) == capsuleFieldType.empty.index) {
      wi++;
    } else {
      wi++;
      xdataLength = bdata.getUint32(wi);
      wi = wi + 4;
      for (int i = 0; i < xdataLength; i++) {
        _xxData[i] = bdata.getUint8(wi + i);
      }
      wi += xdataLength;
    }

    return ModeratorCapsule(
        kdataLength != 0 ? _katamari.sublist(0, kdataLength) : Uint8List(0),
        _threadEntries.sublist(0, tdataLength),
        Uint8List(0),
        _xxWants.sublist(0, xwdataLength),
        _xxData.sublist(0, xdataLength));
  }

  Uint8List get binary {
    Uint8List b = Uint8List(99420000);
    var bdata = new ByteData.view(b.buffer);
    //first word is lenght, then 4 filler byts
    int wi = 6;

    if (katamari.isEmpty) {
      bdata.setUint8(wi++, capsuleFieldType.empty.index);
    } else {
      bdata.setUint8(wi++, capsuleFieldType.katamari.index);
      bdata.setUint16(wi, katamari.lengthInBytes);
      wi += 2;
      katamari.forEach((element) {
        bdata.setUint8(wi++, element);
      });
    }
    if (threadEntries.isEmpty) {
      bdata.setUint8(wi++, capsuleFieldType.empty.index);
    } else {
      bdata.setUint8(wi++, capsuleFieldType.threadEntries.index);
      bdata.setUint16(wi, threadEntries.lengthInBytes);
      wi += 2;
      threadEntries.forEach((element) {
        bdata.setUint8(wi++, element);
      });
    }
    if (groupEntries.isEmpty) {
      bdata.setUint8(wi++, capsuleFieldType.empty.index);
    } else {
      bdata.setUint8(wi++, capsuleFieldType.groupEntries.index);
      bdata.setUint16(wi, groupEntries.lengthInBytes);
      wi += 2;
      groupEntries.forEach((element) {
        bdata.setUint8(wi++, element);
      });
    }
    if (xxWants.isEmpty) {
      bdata.setUint8(wi++, capsuleFieldType.empty.index);
    } else {
      bdata.setUint8(wi++, capsuleFieldType.xxWant.index);
      bdata.setUint16(wi, xxWants.lengthInBytes);
      wi += 2;
      xxWants.forEach((element) {
        bdata.setUint8(wi++, element);
      });
    }
    if (xxData.isEmpty) {
      bdata.setUint8(wi++, capsuleFieldType.empty.index);
    } else {
      bdata.setUint8(wi++, capsuleFieldType.xxData.index);
      bdata.setUint32(wi, xxData.lengthInBytes);
      wi += 4;
      xxData.forEach((element) {
        bdata.setUint8(wi++, element);
      });
    }
    wi += 16; //wiggle room
    bdata.setUint32(0, wi);
    return b.sublist(0, wi);
  }
}

==> interzone-1.0.24.tar.gz: lib/moderator/ModeratorCapsule.dart <==
import 'dart:convert';
import 'dart:typed_data';
export 'ModeratorCapsuleEntry.dart';
import 'package:hex/hex.dart';

enum capsuleFieldType {
  empty,
  tangle,
  katamari,
  threadEntries,
  groupEntries,
  xxWant,
  xxData
}

class ModeratorCapsule {
  //Uint8List tangle; //MUST have token for POST
  Uint8List katamari; //OPTIONAL token for POST
  Uint8List threadEntries; //singular thread entries
  Uint8List groupEntries; //group entries pointing to thread entries
  Uint8List xxWants; //uint64's of missing xx
  Uint8List xxData; //bin data of xx to satisfy
  //for stripping dup entries
  bool get isEmpty =>
      katamari.isEmpty &&
      threadEntries.isEmpty &&
      xxWants.isEmpty &&
      xxData.isEmpty;
  bool get isDuplicate =>
      katamari.lengthInBytes == 1 &&
      threadEntries.lengthInBytes == 1 &&
      groupEntries.lengthInBytes == 1;

  ModeratorCapsule(this.katamari, this.threadEntries, this.groupEntries,
      this.xxWants, this.xxData);
  factory ModeratorCapsule.duplicate() {
    return ModeratorCapsule(
        Uint8List(1), Uint8List(1), Uint8List(1), Uint8List(1), Uint8List(1));
  }
  factory ModeratorCapsule.fromBinary(Uint8List data) {
    var bdata = new ByteData.view(data.buffer);
    data.buffer.asUint8List().toList();
    int dataLength = bdata.getUint32(0);
    if (dataLength != data.lengthInBytes) {
      return ModeratorCapsule(
          Uint8List(0), Uint8List(0), Uint8List(0), Uint8List(0), Uint8List(0));
    }
    var wi = 6; //skipping tangloid
    Uint8List _tangle = Uint8List(0); //MUST have key to talk to moderator
    Uint8List _katamari =
        Uint8List(dataLength); //currently OPTIONAL key for your requests
    Uint8List _threadEntries =
        Uint8List(10065536); //single thread entries with keys
    Uint8List _groupEntries =
        Uint8List(10065536); //single group entries with keys
    Uint8List _xxWants = Uint8List(8089); //single group entries with keys
    Uint8List _xxData =
        Uint8List(data.buffer.lengthInBytes); //single group entries with keys

    //tangle is a must
    /*int tadataLength = 0;
    if (bdata.getInt8(wi) == capsuleFieldType.empty) {
      wi++;
    } else {
      wi++;
      tadataLength = bdata.getInt16(wi);
      wi = wi + 2;
      for (int i = 0; i < tadataLength; i++) {
        _tangle.add(data[wi + i]);
      }
    }*/
    //katamari

    print(bdata.lengthInBytes.toString());
    int kdataLength = 0;
    if (bdata.getUint8(wi) == capsuleFieldType.empty.index) {
      wi++;
    } else {
      wi++;
      kdataLength = bdata.getUint16(wi);
      wi = wi + 2;
      var moo = wi + kdataLength;
      //print(moo);
      //print(bdata.lengthInBytes);
      try {
        for (int i = 0; i < kdataLength; i++) {
          //_katamari.add(data[wi + i]);
          _katamari[i] = bdata.getUint8(wi + i);
        }
      } catch (e) {
        print('missing byte');
        print(moo);
        print(bdata.lengthInBytes);
      }
      wi += kdataLength;
    }
    //threads 5
    var tdataLength = 0;
    if (bdata.getUint8(wi) == capsuleFieldType.empty.index) {
      wi++;
    } else {
      wi++;
      tdataLength = bdata.getUint16(wi);
      if (tdataLength > bdata.lengthInBytes) {
        print('trap');
      }
      if (tdataLength > _threadEntries.lengthInBytes) {
        print('trap');
      }
      wi = wi + 2;
      for (int i = 0; i < tdataLength; i++) {
        _threadEntries[i] = bdata.getUint8(wi + i);
        //_threadEntries.add(data[wi + i]);
      }
      wi += tdataLength;
    }
    //groups 6
    if (bdata.getUint8(wi) == capsuleFieldType.empty.index) {
      wi++;
    } else {
      wi++;
      int xdataLength = bdata.getUint16(wi);
      wi = wi + 2;
      for (int i = 0; i < xdataLength; i++) {
        _groupEntries[i] = bdata.getUint8(wi + i);
      }
      wi += xdataLength;
    }
    //XXWANTS 7
    int xwdataLength = 0;
    if (wi + 8 > bdata.lengthInBytes) {
      return ModeratorCapsule(
          kdataLength != 0 ? _katamari.sublist(0, kdataLength) : Uint8List(0),
          _threadEntries.sublist(0, tdataLength),
          Uint8List(0),
          Uint8List(0),
          Uint8List(0));
    }
    if (bdata.getUint8(wi) == capsuleFieldType.empty.index) {
      wi++;
    } else {
      wi++;
      xwdataLength = bdata.getUint16(wi);
      wi = wi + 2;
      for (int i = 0; i < xwdataLength; i++) {
        _xxWants[i] = bdata.getUint8(wi + i);
      }
      wi += xwdataLength;
    }
    int xdataLength = 0;
    if (bdata.getUint8(wi) == capsuleFieldType.empty.index) {
      wi++;
    } else {
      wi++;
      xdataLength = bdata.getUint32(wi);
      wi = wi + 4;
      for (int i = 0; i < xdataLength; i++) {
        _xxData[i] = bdata.getUint8(wi + i);
      }
      wi += xdataLength;
    }

    return ModeratorCapsule(
        kdataLength != 0 ? _katamari.sublist(0, kdataLength) : Uint8List(0),
        _threadEntries.sublist(0, tdataLength),
        Uint8List(0),
        _xxWants.sublist(0, xwdataLength),
        _xxData.sublist(0, xdataLength));
  }

  Uint8List get binary {
    Uint8List b = Uint8List(99420000);
    var bdata = new ByteData.view(b.buffer);
    //first word is lenght, then 4 filler byts
    int wi = 6;

    if (katamari.isEmpty) {
      bdata.setUint8(wi++, capsuleFieldType.empty.index);
    } else {
      bdata.setUint8(wi++, capsuleFieldType.katamari.index);
      bdata.setUint16(wi, katamari.lengthInBytes);
      wi += 2;
      katamari.forEach((element) {
        bdata.setUint8(wi++, element);
      });
    }
    if (threadEntries.isEmpty) {
      bdata.setUint8(wi++, capsuleFieldType.empty.index);
    } else {
      bdata.setUint8(wi++, capsuleFieldType.threadEntries.index);
      bdata.setUint16(wi, threadEntries.lengthInBytes);
      wi += 2;
      threadEntries.forEach((element) {
        bdata.setUint8(wi++, element);
      });
    }
    if (groupEntries.isEmpty) {
      bdata.setUint8(wi++, capsuleFieldType.empty.index);
    } else {
      bdata.setUint8(wi++, capsuleFieldType.groupEntries.index);
      bdata.setUint16(wi, groupEntries.lengthInBytes);
      wi += 2;
      groupEntries.forEach((element) {
        bdata.setUint8(wi++, element);
      });
    }
    if (xxWants.isEmpty) {
      bdata.setUint8(wi++, capsuleFieldType.empty.index);
    } else {
      bdata.setUint8(wi++, capsuleFieldType.xxWant.index);
      bdata.setUint16(wi, xxWants.lengthInBytes);
      wi += 2;
      xxWants.forEach((element) {
        bdata.setUint8(wi++, element);
      });
    }
    if (xxData.isEmpty) {
      bdata.setUint8(wi++, capsuleFieldType.empty.index);
    } else {
      bdata.setUint8(wi++, capsuleFieldType.xxData.index);
      bdata.setUint32(wi, xxData.lengthInBytes);
      wi += 4;
      for (var element in xxData) {
        bdata.setUint8(wi++, element);
      }
    }
    wi += 16; //wiggle room
    bdata.setUint32(0, wi);
    return b.sublist(0, wi);
  }
}

class ModeratorCapsuleItemAttachment {
  int length;
  String xxHash;
  String cid;
  int get xxHashInt {
    if (xxHash.isEmpty) return 0;
    final sma = HEX.decode(xxHash) as Uint8List;
    var b = new ByteData.view(sma.buffer);
    return b.getInt64(0);
  }

  ModeratorCapsuleItemAttachment(this.length, this.xxHash, this.cid);
  Uint8List get binary {
    Uint8List b = Uint8List(17);
    var bdata = new ByteData.view(b.buffer);
    int sds = 0;
    //b[0] = 16; //att heaeder is tis long
    Uint8List _atl = Uint8List(8)
      ..buffer.asByteData().setUint64(0, length, Endian.big);

    for (var i in _atl) {
      bdata.setUint8(sds++, i);
    }
    for (var i in HEX.decode(xxHash)) {
      bdata.setUint8(sds++, i);
    }
    //last byte 0 no cid or cid len
    return b;
    /*if (cid.isEmpty) {
      bdata.setUint8(16, 0);
      return b.sublist(0, 17);
    }
    var c = utf8.encode(cid) as Uint8List;
    bdata.setUint8(16, c.lengthInBytes);
    var wi = 17;
    for (var cc in c) {
      bdata.setUint8(wi++, cc);
    }
    return b.sublist(0, wi); */
  }

  factory ModeratorCapsuleItemAttachment.fromBinary(Uint8List data) {
    var bdata = new ByteData.view(data.buffer);
    int bleng = bdata.getUint32(0);
    String xx = HEX.encode(data.sublist(4, 12));
    /*int cidl = bdata.getUint8(16);
    if (cidl == 0) {
      return ModeratorCapsuleItemAttachment(
          bleng, base64Encode(xx.toString().codeUnits), '');
    }
    String cid = utf8.decode(data.sublist(17, 17 + cidl));
    */
    //DANGER
    return ModeratorCapsuleItemAttachment(
        bleng, HEX.encode(data.sublist(8, 16)), '');
  }
}
